@@ -1,4 +1,5 @@
 import base64
+import logging
 from abc import ABC, abstractmethod
 from collections import defaultdict
 from typing import Any, Callable, DefaultDict, Dict, Final, List, Optional, Set, Tuple
@@ -19,13 +20,17 @@ from meta_memcache.protocol import (
     Miss,
     NotStored,
     ReadResponse,
+    ServerVersion,
     Success,
     TokenFlag,
     Value,
     WriteResponse,
+    encode_size,
 )
 from meta_memcache.settings import MAX_KEY_SIZE
 
+_log: logging.Logger = logging.getLogger(__name__)
+
 
 class BaseCachePool(ABC):
     def __init__(
@@ -45,6 +50,8 @@ class BaseCachePool(ABC):
     def _encode_key(self, key: Key) -> Tuple[bytes, bool]:
         if key.is_unicode or len(key.key) > MAX_KEY_SIZE:
             return base64.b64encode(self._binary_key_encoding_fn(key)), True
+        elif " " in key.key:
+            raise ValueError(f"Invalid key {key}")
         else:
             return key.key.encode("ascii"), False
 
@@ -56,11 +63,12 @@ class BaseCachePool(ABC):
         flags: Optional[Set[Flag]] = None,
         int_flags: Optional[Dict[IntFlag, int]] = None,
         token_flags: Optional[Dict[TokenFlag, bytes]] = None,
+        version: ServerVersion = ServerVersion.STABLE,
     ) -> bytes:
         encoded_key, is_binary = self._encode_key(key)
         cmd = [command.value, encoded_key]
         if size is not None:
-            cmd.append(str(size).encode("ascii"))
+            cmd.append(encode_size(size, version=version))
         cmd_flags = []
         if is_binary:
             cmd_flags.append(Flag.BINARY.value)
@@ -173,11 +181,20 @@ class BaseCachePool(ABC):
             flags=flags,
             int_flags=int_flags,
             token_flags=token_flags,
+            version=conn.get_version(),
         )
+        # write meta commands with NOREPLY can potentially return errors
+        # they are not fully silent, so we need to add a no-op to the wire.
+        with_noop = (
+            command != MetaCommand.META_GET
+            and flags is not None
+            and Flag.NOREPLY in flags
+        )
+
         if value:
-            conn.sendall(cmd + value + ENDL)
+            conn.sendall(cmd + value + ENDL, with_noop=with_noop)
         else:
-            conn.sendall(cmd)
+            conn.sendall(cmd, with_noop=with_noop)
 
     def _conn_recv_response(
         self,
@@ -191,11 +208,15 @@ class BaseCachePool(ABC):
             return Success(flags=set([Flag.NOREPLY]))
         result = conn.get_response()
         if isinstance(result, Value):
-            # TODO: Confirm this works for empty values!
             data = conn.get_value(result.size)
             if result.size > 0:
                 encoding_id = result.int_flags.get(IntFlag.CLIENT_FLAG, 0)
-                result.value = self._serializer.unserialize(data, encoding_id)
+                try:
+                    result.value = self._serializer.unserialize(data, encoding_id)
+                except Exception:
+                    _log.exception("Error unserializing value")
+                    result = Miss()
+
         return result
 
     def meta_multiget(
