@@ -1,11 +1,11 @@
 from dataclasses import dataclass
-from enum import Enum
+from enum import Enum, IntEnum
 from typing import Any, Dict, NamedTuple, Optional, Set, Union
 
 ENDL = b"\r\n"
+NOOP: bytes = b"mn" + ENDL
 ENDL_LEN = 2
 SPACE: int = ord(" ")
-MIN_HEADER_SIZE = 4
 
 
 class Key(NamedTuple):
@@ -50,7 +50,8 @@ class IntFlag(Enum):
     HIT_AFTER_WRITE = b"h"
     MA_INITIAL_VALUE = b"J"
     MA_DELTA_VALUE = b"D"
-    CAS_TOKEN = b"c"
+    RETURNED_CAS_TOKEN = b"c"
+    CAS_TOKEN = b"C"
 
 
 class TokenFlag(Enum):
@@ -133,3 +134,28 @@ WriteResponse = Union[Success, NotStored, Conflict, Miss]
 
 
 Blob = Union[bytes, bytearray, memoryview]
+
+
+class ServerVersion(IntEnum):
+    """
+    If more versions with breaking changes are
+    added, bump stable to the next int. Code
+    will be able to use > / < / = to code
+    the behavior of the different versions.
+    """
+
+    AWS_1_6_6 = 1
+    STABLE = 2
+
+
+def get_store_success_response_header(version: ServerVersion) -> bytes:
+    if version == ServerVersion.AWS_1_6_6:
+        return b"OK"
+    return b"HD"
+
+
+def encode_size(size: int, version: ServerVersion) -> bytes:
+    if version == ServerVersion.AWS_1_6_6:
+        return b"S" + str(size).encode("ascii")
+    else:
+        return str(size).encode("ascii")
